-- Junkyard
-- by Hansi-TWZ
-- DE333-SERVER.EU
-- Credits to MewMew (Comfy)

local table_insert = table.insert
local math_random = math.random

local tile_replacements = {
	["dirt-3"] = "grass-1",
	["dirt-2"] = "grass-2",
	["dirt-1"] = "grass-3",
	["sand-1"] = "grass-4",
	["sand-2"] = "grass-1",
	["sand-3"] = "grass-2",
	["red-desert-1"] = "grass-3",
	["red-desert-2"] = "grass-4",
	["red-desert-3"] = "grass-1",
	["red-desert-0"] = "grass-2",
	["dry-dirt"] = "grass-3"
}

local entity_replacements = {
	["tree-01"] = "dry-tree",
	["tree-02"] = "tree-06-brown",
	["tree-03"] = "dead-grey-trunk",
	["tree-04"] = "dry-tree",	
	["tree-05"] = "dead-grey-trunk",	
	["tree-06"] = "dry-hairy-tree",	
	["tree-07"] = "dead-grey-trunk",	
	["tree-08"] = "dead-grey-trunk",	
	["tree-09"] = "dry-tree",
	["tree-02-red"] = "dry-hairy-tree",	
	["tree-08-brown"] = "tree-06-brown",	
	["tree-09-brown"] = "tree-06-brown",	
	["tree-09-red"] = "tree-06-brown",
	["iron-ore"] = "mineable-wreckage",
	["copper-ore"] = "mineable-wreckage",
	["coal"] = "mineable-wreckage",
	["stone"] = "mineable-wreckage",
	["uranium-ore"] = "mineable-wreckage"
 }
 
local construction = {"assembling-machine-1", "assembling-machine-2", "assembling-machine-3"}
local furnace = {"stone-furnace", "steel-furnace"}
local construction_chest = {"wooden-chest", "iron-chest", "steel-chest","logistic-chest-passive-provider"}
local chemical_chest = {"wooden-chest", "iron-chest", "steel-chest","logistic-chest-passive-provider", "logistic-chest-buffer", "logistic-chest-requester"}
local atomic_chest = {"logistic-chest-passive-provider", "logistic-chest-buffer", "logistic-chest-requester"}
local robo_chest = {"logistic-chest-buffer", "logistic-chest-requester"}
local inserter = {"inserter", "fast-inserter", "filter-inserter", "stack-inserter", "stack-filter-inserter"}
local electric_pole_small = {"small-electric-pole", "medium-electric-pole"}
local electric_pole_big = {"big-electric-pole", "substation"}
local defense = {"gun-turret", "flamethrower-turret", "radar"}
local defense_rare = {"gun-turret", "laser-turret"}
local lab_chest = {"wooden-chest", "iron-chest", "steel-chest"}
local mixed = {"assembling-machine-1", "assembling-machine-2","stone-furnace", "steel-furnace", "small-electric-pole", "medium-electric-pole","inserter", "fast-inserter", "filter-inserter", "chemical-plant", "solar-panel", "accumulator", "steam-engine", "storage-tank", "arithmetic-combinator", "decider-combinator", "constant-combinator", "power-switch", "programmable-speaker", "pumpjack", "electric-mining-drill", "pump", "radar"}
local mixed_rare = {"assembling-machine-3","stack-inserter", "stack-filter-inserter", "roboport", "electric-furnace", "steam-turbine", "heat-exchanger", "nuclear-reactor", "beacon"}


local function process_entity(e)
	if not e.valid then return end
	if entity_replacements[e.name] then		
		if e.type == "tree" then
			if math_random(1,2) == 1 then
				e.surface.create_entity({name = entity_replacements[e.name], position = e.position})
			end
		else
			e.surface.create_entity({name = entity_replacements[e.name], position = e.position})
		end
		e.destroy()
		return
	end
	if e.type == "unit-spawner" then			
		for _, wreck in pairs (e.surface.find_entities_filtered({area = {{e.position.x - 4, e.position.y - 4},{e.position.x + 4, e.position.y + 4}}, name = "mineable-wreckage"})) do
			if wreck.valid then wreck.destroy() end
		end
		return
	end
end

function Junkyard(event)
	local surface = event.surface
	local left_top = event.area.left_top
	local tiles = {}
	local entities = {}
	
	local raffle_cons = {}
	local loot_cons = {						
		
		{{name = "iron-gear-wheel", count = math_random(80,100)}, weight = 3, evolution_min = 0.0, evolution_max = 0.3},
		{{name = "copper-cable", count = math_random(100,200)}, weight = 3, evolution_min = 0.0, evolution_max = 0.3},
		{{name = "engine-unit", count = math_random(16,32)}, weight = 2, evolution_min = 0.1, evolution_max = 0.5},
		{{name = "electric-engine-unit", count = math_random(16,32)}, weight = 2, evolution_min = 0.4, evolution_max = 0.8},
		{{name = "battery", count = math_random(40,80)}, weight = 2, evolution_min = 0.3, evolution_max = 0.8},
		{{name = "advanced-circuit", count = math_random(40,80)}, weight = 3, evolution_min = 0.4, evolution_max = 1},
		{{name = "electronic-circuit", count = math_random(100,200)}, weight = 3, evolution_min = 0.0, evolution_max = 0.4},
		{{name = "processing-unit", count = math_random(30,60)}, weight = 3, evolution_min = 0.7, evolution_max = 1},
		{{name = "lubricant-barrel", count = math_random(4,10)}, weight = 1, evolution_min = 0.3, evolution_max = 0.5},
		{{name = "rocket-fuel", count = math_random(4,10)}, weight = 2, evolution_min = 0.3, evolution_max = 0.7},		
		{{name = "steel-plate", count = math_random(50,100)}, weight = 2, evolution_min = 0.1, evolution_max = 0.3},
		{{name = "burner-inserter", count = math_random(4,8)}, weight = 3, evolution_min = 0.0, evolution_max = 0.1},
		{{name = "inserter", count = math_random(4,8)}, weight = 3, evolution_min = 0.0, evolution_max = 0.4},
		{{name = "long-handed-inserter", count = math_random(4,8)}, weight = 3, evolution_min = 0.0, evolution_max = 0.4},		
		{{name = "fast-inserter", count = math_random(4,8)}, weight = 3, evolution_min = 0.1, evolution_max = 1},
		{{name = "filter-inserter", count = math_random(4,8)}, weight = 1, evolution_min = 0.2, evolution_max = 1},		
		{{name = "stack-filter-inserter", count = math_random(2,4)}, weight = 1, evolution_min = 0.4, evolution_max = 1},
		{{name = "stack-inserter", count = math_random(2,4)}, weight = 3, evolution_min = 0.3, evolution_max = 1},				
		{{name = "small-electric-pole", count = math_random(8,16)}, weight = 3, evolution_min = 0.0, evolution_max = 0.3},
		{{name = "medium-electric-pole", count = math_random(4,8)}, weight = 3, evolution_min = 0.2, evolution_max = 1},		
		{{name = "wooden-chest", count = math_random(16,24)}, weight = 3, evolution_min = 0.0, evolution_max = 0.2},
		{{name = "iron-chest", count = math_random(4,8)}, weight = 3, evolution_min = 0.1, evolution_max = 0.4},
		{{name = "steel-chest", count = math_random(4,8)}, weight = 3, evolution_min = 0.3, evolution_max = 1},		
		{{name = "small-lamp", count = math_random(8,16)}, weight = 3, evolution_min = 0.1, evolution_max = 0.3},
		{{name = "rail", count = math_random(50,75)}, weight = 3, evolution_min = 0.1, evolution_max = 0.6},
		{{name = "heat-pipe", count = math_random(8,12)}, weight = 2, evolution_min = 0.5, evolution_max = 1},
		{{name = "arithmetic-combinator", count = math_random(8,16)}, weight = 1, evolution_min = 0.1, evolution_max = 1},
		{{name = "constant-combinator", count = math_random(8,16)}, weight = 1, evolution_min = 0.1, evolution_max = 1},
		{{name = "decider-combinator", count = math_random(8,16)}, weight = 1, evolution_min = 0.1, evolution_max = 1},
		{{name = "power-switch", count = math_random(2,4)}, weight = 1, evolution_min = 0.1, evolution_max = 1},		
		{{name = "programmable-speaker", count = math_random(2,4)}, weight = 1, evolution_min = 0.1, evolution_max = 1},
		{{name = "green-wire", count = math_random(50,100)}, weight = 1, evolution_min = 0.1, evolution_max = 1},
		{{name = "red-wire", count = math_random(50,100)}, weight = 1, evolution_min = 0.1, evolution_max = 1},
		{{name = "express-transport-belt", count = math_random(25,75)}, weight = 3, evolution_min = 0.5, evolution_max = 1},
		{{name = "express-underground-belt", count = math_random(4,8)}, weight = 3, evolution_min = 0.5, evolution_max = 1},		
		{{name = "express-splitter", count = math_random(2,4)}, weight = 3, evolution_min = 0.5, evolution_max = 1},
		{{name = "fast-transport-belt", count = math_random(25,75)}, weight = 3, evolution_min = 0.2, evolution_max = 0.7},
		{{name = "fast-underground-belt", count = math_random(4,8)}, weight = 3, evolution_min = 0.2, evolution_max = 0.7},
		{{name = "fast-splitter", count = math_random(2,4)}, weight = 3, evolution_min = 0.2, evolution_max = 0.3},
		{{name = "transport-belt", count = math_random(25,75)}, weight = 3, evolution_min = 0, evolution_max = 0.3},
		{{name = "underground-belt", count = math_random(4,8)}, weight = 3, evolution_min = 0, evolution_max = 0.3},
		{{name = "splitter", count = math_random(2,4)}, weight = 3, evolution_min = 0, evolution_max = 0.3},		
		{{name = "pipe", count = math_random(40,50)}, weight = 3, evolution_min = 0.0, evolution_max = 0.3},
		{{name = "pipe-to-ground", count = math_random(8,16)}, weight = 1, evolution_min = 0.2, evolution_max = 0.5},
		{{name = "pump", count = math_random(1,4)}, weight = 1, evolution_min = 0.3, evolution_max = 0.8},
		{{name = "rail-signal", count = math_random(8,16)}, weight = 2, evolution_min = 0.2, evolution_max = 0.8},
		{{name = "rail-chain-signal", count = math_random(8,16)}, weight = 2, evolution_min = 0.2, evolution_max = 0.8},		
		{{name = "stone-wall", count = math_random(25,75)}, weight = 1, evolution_min = 0.1, evolution_max = 0.5},
		{{name = "gate", count = math_random(4,8)}, weight = 1, evolution_min = 0.1, evolution_max = 0.5},
		{{name = "train-stop", count = math_random(1,2)}, weight = 1, evolution_min = 0.2, evolution_max = 0.7},
		{{name = "express-loader", count = math_random(1,2)}, weight = 1, evolution_min = 0.5, evolution_max = 1},
		{{name = "fast-loader", count = math_random(1,2)}, weight = 1, evolution_min = 0.2, evolution_max = 0.7},
		{{name = "loader", count = math_random(1,2)}, weight = 1, evolution_min = 0.0, evolution_max = 0.5}	
	}
	
	local raffle_chem = {}
	local loot_chem = {						
		{{name = "explosives", count = math_random(25,50)}, weight = 1, evolution_min = 0.2, evolution_max = 0.6},
		{{name = "battery", count = math_random(40,80)}, weight = 2, evolution_min = 0.2, evolution_max = 1.0},
		{{name = "plastic-bar", count = math_random(60,120)}, weight = 2, evolution_min = 0.3, evolution_max = 1.0},
		{{name = "solid-fuel", count = math_random(25,40)}, weight = 2, evolution_min = 0.2, evolution_max = 1.0},
		{{name = "sulfur", count = math_random(60,120)}, weight = 2, evolution_min = 0.2, evolution_max = 1.0}
	}
	
	local raffle_nuclear = {}
	local loot_nuclear = {						
		
		{{name = "used-up-uranium-fuel-cell", count = math_random(1,8)}, weight = 2, evolution_min = 0.2, evolution_max = 1.0},
		{{name = "uranium-fuel-cell", count = math_random(1,4)}, weight = 2, evolution_min = 0.3, evolution_max = 1.0},
		{{name = "uranium-238", count = math_random(25,40)}, weight = 2, evolution_min = 0.2, evolution_max = 1.0},
		{{name = "uranium-235", count = math_random(25,40)}, weight = 2, evolution_min = 0.2, evolution_max = 1.0},
		{{name = "nuclear-fuel", count = math_random(1,4)}, weight = 2, evolution_min = 0.2, evolution_max = 1.0}
	}
	
	local raffle_atomic = {}
	local loot_atomic = {						
		
		{{name = "used-up-uranium-fuel-cell", count = math_random(1,8)}, weight = 2, evolution_min = 0.2, evolution_max = 1.0},
		{{name = "uranium-fuel-cell", count = math_random(1,4)}, weight = 2, evolution_min = 0.3, evolution_max = 1.0}
	}
	
	local raffle_robo = {}
	local loot_robo = {						
		
		{{name = "construction-robot", count = math_random(12,32)}, weight = 2, evolution_min = 0.0, evolution_max = 1.0},
		{{name = "logistic-robot", count = math_random(8,24)}, weight = 2, evolution_min = 0.0, evolution_max = 1.0}
	}
	
	local raffle_lab = {}
	local loot_lab = {						
		
		{{name = "automation-science-pack", count = math_random(100,200)}, weight = 2, evolution_min = 0.0, evolution_max = 1.0},
		{{name = "logistic-science-pack", count = math_random(100,200)}, weight = 2, evolution_min = 0.0, evolution_max = 1.0},
		{{name = "military-science-pack", count = math_random(100,200)}, weight = 2, evolution_min = 0.0, evolution_max = 1.0},
		{{name = "chemical-science-pack", count = math_random(100,200)}, weight = 2, evolution_min = 0.2, evolution_max = 1.0},
		{{name = "production-science-pack", count = math_random(100,200)}, weight = 2, evolution_min = 0.4, evolution_max = 1.0},
		{{name = "utility-science-pack", count = math_random(100,200)}, weight = 2, evolution_min = 0.6, evolution_max = 1.0},
		{{name = "space-science-pack", count = math_random(800,2000)}, weight = 2, evolution_min = 0.8, evolution_max = 1.0}
	}
	
	for x = 0, 31, 1 do
		for y = 0, 31, 1 do
			local tile_to_insert = false
			local pos = {x = left_top.x + x, y = left_top.y + y}
											
			local tile = surface.get_tile(pos)
			if tile_replacements[tile.name] then
				table_insert(tiles, {name = tile_replacements[tile.name], position = pos})
			end
		end
	end
	surface.set_tiles(tiles, true)
	for _, e in pairs(surface.find_entities_filtered({area = event.area})) do
		process_entity(e)		
	end
	
	for _, _x in pairs(event.surface.find_entities_filtered({area = event.area, name = {"rock-big"}})) do
		local pos = _x.position
		if math_random(1,96) == 1 then
			local wreck = event.surface.create_entity{name = construction[math.random(1,#construction)], position = _x.position, force = "neutral"}
			local wreck2 = event.surface.create_entity{name = inserter[math.random(1,#inserter)], position = {_x.position.x-0, _x.position.y+2}, force = "neutral"}
			local pole = event.surface.create_entity{name = electric_pole_small[math.random(1,#electric_pole_small)], position = {_x.position.x-1, _x.position.y+3}, force = "neutral"}
			for _, t in pairs (loot_cons) do
				for x = 1, t.weight, 1 do
					table.insert(raffle_cons, t[1])
				end
			end
			local wreck_chest = event.surface.create_entity{name = construction_chest[math.random(1,#construction_chest)], position = {_x.position.x-0, _x.position.y+3}, force = "neutral"}
			for x = 1, math_random(1,1), 1 do
				local loot_cons = raffle_cons[math_random(1,#raffle_cons)]
				wreck_chest.insert(loot_cons)
			end	
		else
			local rock = event.surface.create_entity({name = "rock-big", position = _x.position, force = "neutral"})
		end
		_x.destroy()
	end
	
	for _, _x in pairs(event.surface.find_entities_filtered({area = event.area, name = {"sand-rock-big"}})) do
		local pos = _x.position
		if math_random(1,96) == 1 then
			local wreck = event.surface.create_entity{name = furnace[math.random(1,#furnace)], position = _x.position, force = "neutral"}
			local wreck2 = event.surface.create_entity{name = inserter[math.random(1,#inserter)], position = {_x.position.x-0, _x.position.y+2}, force = "neutral"}
			local wreck3 = event.surface.create_entity{name = furnace[math.random(1,#furnace)], position = {_x.position.x-2, _x.position.y+0}, force = "neutral"}
			local pole = event.surface.create_entity{name = electric_pole_small[math.random(1,#electric_pole_small)], position = {_x.position.x-1, _x.position.y+2}, force = "neutral"}
		else
			local rock = event.surface.create_entity({name = "sand-rock-big", position = _x.position, force = "neutral"})
		end
		_x.destroy()
	end
	
	for _, _x in pairs(event.surface.find_entities_filtered({area = event.area, name = {"rock-huge"}})) do
		local pos = _x.position
		if math_random(1,96) == 1 then
			local wreck = event.surface.create_entity{name = mixed[math.random(1,#mixed)], position = _x.position, force = "neutral"}
		else
			local rock = event.surface.create_entity({name = "rock-huge", position = _x.position, force = "neutral"})
		end
		_x.destroy()
	end
	
	for _, _x in pairs(event.surface.find_entities_filtered({area = event.area, name = {"mineable-wreckage"}})) do
		local pos = _x.position
		if math_random(1,256) == 1 then
			local wreck = event.surface.create_entity{name = mixed[math.random(1,#mixed)], position = _x.position, force = "neutral"}
		else
			local rock = event.surface.create_entity({name = "mineable-wreckage", position = _x.position, force = "neutral"})
		end
		_x.destroy()
	end
	
	for _, _x in pairs(event.surface.find_entities_filtered({area = event.area, name = {"mineable-wreckage"}})) do
		local pos = _x.position
		if math_random(1,4096) == 1 then
			local wreck = event.surface.create_entity{name = mixed_rare[math.random(1,#mixed_rare)], position = _x.position, force = "neutral"}
		else
			local rock = event.surface.create_entity({name = "mineable-wreckage", position = _x.position, force = "neutral"})
		end
		_x.destroy()
	end
	
	for _, _x in pairs(event.surface.find_entities_filtered({area = event.area, name = {"mineable-wreckage"}})) do
		local pos = _x.position
		if math_random(1,1024) == 1 then
			local wreck = event.surface.create_entity{name = electric_pole_small[math.random(1,#electric_pole_small)], position = _x.position, force = "neutral"}
		else
			local rock = event.surface.create_entity({name = "mineable-wreckage", position = _x.position, force = "neutral"})
		end
		_x.destroy()
	end
	
	for _, _x in pairs(event.surface.find_entities_filtered({area = event.area, name = {"mineable-wreckage"}})) do
		local pos = _x.position
		if math_random(1,1024) == 1 then
			local wreck = event.surface.create_entity{name = electric_pole_big[math.random(1,#electric_pole_big)], position = _x.position, force = "neutral"}
		else
			local rock = event.surface.create_entity({name = "mineable-wreckage", position = _x.position, force = "neutral"})
		end
		_x.destroy()
	end
	
	for _, _x in pairs(event.surface.find_entities_filtered({area = event.area, name = {"rock-big"}})) do
		local pos = _x.position
		if math_random(1,128) == 1 then
			local wreck = event.surface.create_entity({name = "chemical-plant", position = _x.position, force = "neutral"})
			local wreck2 = event.surface.create_entity{name = inserter[math.random(1,#inserter)], position = {_x.position.x-0, _x.position.y+2}, force = "neutral"}
			local wreck3 = event.surface.create_entity{name = inserter[math.random(1,#inserter)], position = {_x.position.x-0, _x.position.y-2}, force = "neutral"}
			local pole = event.surface.create_entity{name = electric_pole_small[math.random(1,#electric_pole_small)], position = {_x.position.x-1, _x.position.y+3}, force = "neutral"}
			local wreck_chest2 = event.surface.create_entity{name = chemical_chest[math.random(1,#chemical_chest)], position = {_x.position.x-0, _x.position.y-3}, force = "neutral"}
			for _, t in pairs (loot_chem) do
				for x = 1, t.weight, 1 do
					table.insert(raffle_chem, t[1])
				end
			end
			local wreck_chest = event.surface.create_entity{name = chemical_chest[math.random(1,#chemical_chest)], position = {_x.position.x-0, _x.position.y+3}, force = "neutral"}
			for x = 1, math_random(1,1), 1 do
				local loot_chem = raffle_chem[math_random(1,#raffle_chem)]
				wreck_chest.insert(loot_chem)
			end	
		else
			local rock = event.surface.create_entity({name = "rock-big", position = _x.position, force = "neutral"})
		end
		_x.destroy()
	end
	for _, _x in pairs(event.surface.find_entities_filtered({area = event.area, name = {"sand-rock-big"}})) do
		local pos = _x.position
		if math_random(1,256) == 1 then
			local wreck = event.surface.create_entity({name = "centrifuge", position = _x.position, force = "neutral"})
			local wreck2 = event.surface.create_entity{name = inserter[math.random(1,#inserter)], position = {_x.position.x-0, _x.position.y+2}, force = "neutral"}
			local wreck3 = event.surface.create_entity{name = inserter[math.random(1,#inserter)], position = {_x.position.x-0, _x.position.y-2}, force = "neutral"}
			local pole = event.surface.create_entity{name = electric_pole_small[math.random(1,#electric_pole_small)], position = {_x.position.x-1, _x.position.y+3}, force = "neutral"}
			local wreck_chest2 = event.surface.create_entity{name = chemical_chest[math.random(1,#chemical_chest)], position = {_x.position.x-0, _x.position.y-3}, force = "neutral"}
			for _, t in pairs (loot_nuclear) do
				for x = 1, t.weight, 1 do
					table.insert(raffle_nuclear, t[1])
				end
			end
			local wreck_chest = event.surface.create_entity{name = chemical_chest[math.random(1,#chemical_chest)], position = {_x.position.x-0, _x.position.y+3}, force = "neutral"}
			for x = 1, math_random(1,1), 1 do
				local loot_nuclear = raffle_nuclear[math_random(1,#raffle_nuclear)]
				wreck_chest.insert(loot_nuclear)
			end	
		else
			local rock = event.surface.create_entity({name = "sand-rock-big", position = _x.position, force = "neutral"})
		end
		_x.destroy()
	end
	for _, _x in pairs(event.surface.find_entities_filtered({area = event.area, name = {"rock-huge"}})) do
		local pos = _x.position
		if math_random(1,256) == 1 then
			local wreck = event.surface.create_entity({name = "substation", position = _x.position, force = "neutral"})
			local wreck2 = event.surface.create_entity({name = "solar-panel", position = {_x.position.x-0, _x.position.y+4}, force = "neutral"})
			local wreck3 = event.surface.create_entity({name = "solar-panel", position = {_x.position.x-4, _x.position.y+5}, force = "neutral"})
			local wreck4 = event.surface.create_entity({name = "solar-panel", position = {_x.position.x-0, _x.position.y-6}, force = "neutral"})
			local wreck5 = event.surface.create_entity({name = "accumulator", position = {_x.position.x-5, _x.position.y-6}, force = "neutral"})
			local wreck6 = event.surface.create_entity({name = "small-lamp", position = {_x.position.x-2, _x.position.y-0}, force = "neutral"})
		else
			local rock = event.surface.create_entity({name = "rock-huge", position = _x.position})
		end
		_x.destroy()
	end
	for _, _x in pairs(event.surface.find_entities_filtered({area = event.area, name = {"rock-big"}})) do
		local pos = _x.position
		if math_random(1,256) == 1 then
			local wreck = event.surface.create_entity({name = "substation", position = _x.position, force = "neutral"})
			local wreck2 = event.surface.create_entity({name = "solar-panel", position = {_x.position.x-0, _x.position.y+4}, force = "neutral"})
			local wreck3 = event.surface.create_entity({name = "solar-panel", position = {_x.position.x-4, _x.position.y+5}, force = "neutral"})
			local wreck4 = event.surface.create_entity({name = "solar-panel", position = {_x.position.x-0, _x.position.y-6}, force = "neutral"})
			local wreck5 = event.surface.create_entity({name = "accumulator", position = {_x.position.x-5, _x.position.y-6}, force = "neutral"})
			local wreck6 = event.surface.create_entity({name = "accumulator", position = {_x.position.x-5, _x.position.y-3}, force = "neutral"})
		else
			local rock = event.surface.create_entity({name = "rock-big", position = _x.position})
		end
		_x.destroy()
	end
	for _, _x in pairs(event.surface.find_entities_filtered({area = event.area, name = {"sand-rock-big"}})) do
		local pos = _x.position
		if math_random(1,256) == 1 then
			local wreck = event.surface.create_entity({name = "oil-refinery", position = _x.position, force = "neutral"})
			local wreck2 = event.surface.create_entity({name = "storage-tank", position = {_x.position.x-0, _x.position.y-5}, force = "neutral"})
			local pole = event.surface.create_entity{name = electric_pole_small[math.random(1,#electric_pole_small)], position = {_x.position.x-3, _x.position.y-4}, force = "neutral"}
			local fluids = {"heavy-oil", "light-oil", "petroleum-gas"}
			wreck2.fluidbox[1] = {name = fluids[math.random(1, #fluids)], amount = math.random(15000, 25000)}
		else
			local rock = event.surface.create_entity({name = "sand-rock-big", position = _x.position})
		end
		_x.destroy()
	end
	for _, _x in pairs(event.surface.find_entities_filtered({area = event.area, name = {"rock-huge"}})) do
		local pos = _x.position
		if math_random(1,256) == 1 then
			local wreck = event.surface.create_entity({name = "chemical-plant", position = _x.position, force = "neutral"})
			local wreck2 = event.surface.create_entity({name = "storage-tank", position = {_x.position.x-0, _x.position.y-4}, force = "neutral"})
			local pole = event.surface.create_entity{name = electric_pole_small[math.random(1,#electric_pole_small)], position = {_x.position.x-1, _x.position.y+2}, force = "neutral"}
			local fluids = {"lubricant", "heavy-oil", "light-oil", "petroleum-gas", "sulfuric-acid", "water"}
			wreck2.fluidbox[1] = {name = fluids[math.random(1, #fluids)], amount = math.random(15000, 25000)}
		else
			local rock = event.surface.create_entity({name = "rock-huge", position = _x.position})
		end
		_x.destroy()
	end
	for _, _x in pairs(event.surface.find_entities_filtered({area = event.area, name = {"rock-big"}})) do
		local pos = _x.position
		if math_random(1,256) == 1 then
			local wreck = event.surface.create_entity{name = defense[math.random(1,#defense)], position = {_x.position.x-0, _x.position.y+2}, force = "neutral"}
		else
			local rock = event.surface.create_entity({name = "rock-big", position = _x.position})
		end
		_x.destroy()
	end
	for _, _x in pairs(event.surface.find_entities_filtered({area = event.area, name = {"sand-rock-big"}})) do
		local pos = _x.position
		if math_random(1,1024) == 1 then
			local wreck = event.surface.create_entity{name = defense_rare[math.random(1,#defense_rare)], position = {_x.position.x-0, _x.position.y+2}, force = "neutral"}
		else
			local rock = event.surface.create_entity({name = "sand-rock-big", position = _x.position})
		end
		_x.destroy()
	end
	for _, _x in pairs(event.surface.find_entities_filtered({area = event.area, name = {"biter-spawner"}})) do
		local pos = _x.position
		if math_random(1,256) == 1 then
			local wreck = event.surface.create_entity{name = mixed[math.random(1,#mixed)], position = _x.position, force = "neutral"}
		else
			local biter_nest = event.surface.create_entity({name = "biter-spawner", position = _x.position})
		end
		_x.destroy()
	end
	for _, _x in pairs(event.surface.find_entities_filtered({area = event.area, name = {"biter-spawner"}})) do
		local pos = _x.position
		if math_random(1,512) == 1 then
			local wreck = event.surface.create_entity({name = "nuclear-reactor", position = _x.position, force = "neutral"})
			for _, t in pairs (loot_atomic) do
				for x = 1, t.weight, 1 do
					table.insert(raffle_atomic, t[1])
				end
			end
			local wreck_chest = event.surface.create_entity{name = atomic_chest[math.random(1,#atomic_chest)], position = {_x.position.x-0, _x.position.y+4}, force = "neutral"}
			for x = 1, math_random(1,1), 1 do
				local loot_atomic = raffle_atomic[math_random(1,#raffle_atomic)]
				wreck_chest.insert(loot_atomic)
			end	
		else
			local biter_nest = event.surface.create_entity({name = "biter-spawner", position = _x.position})
		end
		_x.destroy()
	end
	for _, _x in pairs(event.surface.find_entities_filtered({area = event.area, name = {"spitter-spawner"}})) do
		local pos = _x.position
		if math_random(1,512) == 1 then
			local wreck = event.surface.create_entity({name = "roboport", position = _x.position, force = "neutral"})
			for _, t in pairs (loot_robo) do
				for x = 1, t.weight, 1 do
					table.insert(raffle_robo, t[1])
				end
			end
			local wreck_chest = event.surface.create_entity{name = robo_chest[math.random(1,#robo_chest)], position = {_x.position.x-0, _x.position.y+4}, force = "neutral"}
			for x = 1, math_random(1,1), 1 do
				local loot_robo = raffle_robo[math_random(1,#raffle_robo)]
				wreck_chest.insert(loot_robo)
			end	
		else
			local spitter_nest = event.surface.create_entity({name = "spitter-spawner", position = _x.position})
		end
		_x.destroy()
	end
	for _, _x in pairs(event.surface.find_entities_filtered({area = event.area, name = {"biter-spawner"}})) do
		local pos = _x.position
		if math_random(1,512) == 1 then
			local wreck = event.surface.create_entity({name = "lab", position = _x.position, force = "neutral"})
			local wreck2 = event.surface.create_entity{name = inserter[math.random(1,#inserter)], position = {_x.position.x-0, _x.position.y+2}, force = "neutral"}
			local pole = event.surface.create_entity{name = electric_pole_small[math.random(1,#electric_pole_small)], position = {_x.position.x-1, _x.position.y+3}, force = "neutral"}
			for _, t in pairs (loot_lab) do
				for x = 1, t.weight, 1 do
					table.insert(raffle_lab, t[1])
				end
			end
			local wreck_chest = event.surface.create_entity{name = lab_chest[math.random(1,#lab_chest)], position = {_x.position.x-0, _x.position.y+3}, force = "neutral"}
			for x = 1, math_random(1,1), 1 do
				local loot_lab = raffle_lab[math_random(1,#raffle_lab)]
				wreck_chest.insert(loot_lab)
			end	
		else
			local biter_nest = event.surface.create_entity({name = "biter-spawner", position = _x.position})
		end
		_x.destroy()
	end
	for _, _x in pairs(event.surface.find_entities_filtered({area = event.area, name = {"dry-tree"}})) do
		local pos = _x.position
		if math_random(1,4) == 1 then
			local wreck = event.surface.create_entity({name = "mineable-wreckage", position = {_x.position.x-0 + math_random(1,4), _x.position.y+2 + math_random(1,4)}, force = "neutral"})
			local wreck2 = event.surface.create_entity({name = "mineable-wreckage", position = {_x.position.x-1 + math_random(1,4), _x.position.y+4 + math_random(1,4)}, force = "neutral"})
			local wreck3 = event.surface.create_entity({name = "mineable-wreckage", position = {_x.position.x-3 + math_random(1,4), _x.position.y+1 + math_random(1,4)}, force = "neutral"})
			local wreck4 = event.surface.create_entity({name = "mineable-wreckage", position = {_x.position.x-3 + math_random(1,4), _x.position.y+2 + math_random(1,4)}, force = "neutral"})
			local wreck5 = event.surface.create_entity({name = "mineable-wreckage", position = {_x.position.x+2 + math_random(1,4), _x.position.y+2 + math_random(1,4)}, force = "neutral"})
			local wreck6 = event.surface.create_entity({name = "mineable-wreckage", position = {_x.position.x+2 + math_random(1,4), _x.position.y+4 + math_random(1,4)}, force = "neutral"})	
		end
	end
	for _, _x in pairs(event.surface.find_entities_filtered({area = event.area, name = {"tree-06-brown"}})) do
		local pos = _x.position
		if math_random(1,4) == 1 then
			local wreck = event.surface.create_entity({name = "mineable-wreckage", position = {_x.position.x-0 + math_random(1,4), _x.position.y+2 + math_random(1,4)}, force = "neutral"})
			local wreck2 = event.surface.create_entity({name = "mineable-wreckage", position = {_x.position.x-1 + math_random(1,4), _x.position.y+4 + math_random(1,4)}, force = "neutral"})
			local wreck3 = event.surface.create_entity({name = "mineable-wreckage", position = {_x.position.x-3 + math_random(1,4), _x.position.y+1 + math_random(1,4)}, force = "neutral"})
			local wreck4 = event.surface.create_entity({name = "mineable-wreckage", position = {_x.position.x-3 + math_random(1,4), _x.position.y+2 + math_random(1,4)}, force = "neutral"})
			local wreck5 = event.surface.create_entity({name = "mineable-wreckage", position = {_x.position.x+2 + math_random(1,4), _x.position.y+2 + math_random(1,4)}, force = "neutral"})
			local wreck6 = event.surface.create_entity({name = "mineable-wreckage", position = {_x.position.x+2 + math_random(1,4), _x.position.y+4 + math_random(1,4)}, force = "neutral"})		
		end
	end
	for _, _x in pairs(event.surface.find_entities_filtered({area = event.area, name = {"dead-grey-trunk"}})) do
		local pos = _x.position
		if math_random(1,4) == 1 then
			local wreck = event.surface.create_entity({name = "mineable-wreckage", position = {_x.position.x-0 + math_random(1,4), _x.position.y+2 + math_random(1,4)}, force = "neutral"})
			local wreck2 = event.surface.create_entity({name = "mineable-wreckage", position = {_x.position.x-1 + math_random(1,4), _x.position.y+4 + math_random(1,4)}, force = "neutral"})
			local wreck3 = event.surface.create_entity({name = "mineable-wreckage", position = {_x.position.x-3 + math_random(1,4), _x.position.y+1 + math_random(1,4)}, force = "neutral"})
			local wreck4 = event.surface.create_entity({name = "mineable-wreckage", position = {_x.position.x-3 + math_random(1,4), _x.position.y+2 + math_random(1,4)}, force = "neutral"})
			local wreck5 = event.surface.create_entity({name = "mineable-wreckage", position = {_x.position.x+2 + math_random(1,4), _x.position.y+2 + math_random(1,4)}, force = "neutral"})
			local wreck6 = event.surface.create_entity({name = "mineable-wreckage", position = {_x.position.x+2 + math_random(1,4), _x.position.y+4 + math_random(1,4)}, force = "neutral"})				
		end
	end
	for _, _x in pairs(event.surface.find_entities_filtered({area = event.area, name = {"cliff"}})) do
		local pos = _x.position
		if math_random(1,6) == 1 then
			local wreck = event.surface.create_entity({name = "mineable-wreckage", position = {_x.position.x-0 + math_random(1,4), _x.position.y+2 + math_random(1,4)}, force = "neutral"})
			local wreck2 = event.surface.create_entity({name = "mineable-wreckage", position = {_x.position.x-1 + math_random(1,4), _x.position.y+4 + math_random(1,4)}, force = "neutral"})
			local wreck3 = event.surface.create_entity({name = "mineable-wreckage", position = {_x.position.x-3 + math_random(1,4), _x.position.y+1 + math_random(1,4)}, force = "neutral"})
			local wreck4 = event.surface.create_entity({name = "mineable-wreckage", position = {_x.position.x-3 + math_random(1,4), _x.position.y+2 + math_random(1,4)}, force = "neutral"})
			local wreck5 = event.surface.create_entity({name = "mineable-wreckage", position = {_x.position.x+2 + math_random(1,4), _x.position.y+2 + math_random(1,4)}, force = "neutral"})
			local wreck6 = event.surface.create_entity({name = "mineable-wreckage", position = {_x.position.x+2 + math_random(1,4), _x.position.y+4 + math_random(1,4)}, force = "neutral"})				
		end
	end
end

function junkyard_chunk_charted(event)
	if not global.chunks_charted then global.chunks_charted = {} end
	local surface = game.surfaces[event.surface_index]
	local position = event.position
	if global.chunks_charted[tostring(position.x) .. tostring(position.y)] then return end
	global.chunks_charted[tostring(position.x) .. tostring(position.y)] = true
	
	local decorative_names = {}
	for k,v in pairs(game.decorative_prototypes) do
		if v.autoplace_specification then
			decorative_names[#decorative_names+1] = k
		end
	end
	surface.regenerate_decorative(decorative_names, {position})	
end