-- control.lua
-- Mar 2019

-- Oarc's Separated Spawn Scenario
--
-- I wanted to create a scenario that allows you to spawn in separate locations
-- From there, I ended up adding a bunch of other minor/major features
--
-- Credit:
--  Tags - Taken from WOGs scenario
--  Rocket Silo - Taken from Frontier as an idea
--
-- Feel free to re-use anything you want. It would be nice to give me credit
-- if you can.



-- To keep the scenario more manageable (for myself) I have done the following:
--      1. Keep all event calls in control.lua (here)
--      2. Put all config options in config.lua and provided an example-config.lua file too.
--      3. Put other stuff into their own files where possible.
--      4. Put all other files into lib folder
--      5. Provided an examples folder for example/recommended map gen settings


-- Generic Utility Includes
require("lib/oarc_utils")

-- Other soft-mod type features.
require("lib/frontier_silo")
require("lib/tag")
require("lib/game_opts")
require("lib/player_list")
require("lib/rocket_launch")
require("lib/regrowth_map")

-- For Philip. I currently do not use this and need to add proper support for
-- commands like this in the future.
-- require("lib/rgcommand")
-- require("lib/helper_commands")

-- Main Configuration File
require("config")
require("DE333/DE333_config")

-- Save all config settings to global table.
require("lib/oarc_global_cfg.lua")

-- Scenario Specific Includes
require("lib/separate_spawns")
require("lib/separate_spawns_guis")

require("lib/oarc_gui_tabs")

-- Create a new surface so we can modify map settings at the start.
GAME_SURFACE_NAME="DE333"

-- DE333
require("DE333/server_config/DE333_server_config")
require("DE333/server_config/server_1")
require("DE333/server_config/server_2")
require("DE333/server_config/server_3")
require("DE333/server_config/server_4")
require("DE333/server_config/server_5")
require("DE333/server_config/server_6")
require("DE333/server_config/server_7")
require("DE333/DE333_utils")


-- I'm reverting my decision to turn the regrowth thing into a mod.
remote.add_interface("oarc_regrowth",
            {area_offlimits_chunkpos = MarkAreaSafeGivenChunkPos,
            area_offlimits_tilepos = MarkAreaSafeGivenTilePos,
            area_removal_tilepos = MarkAreaForRemoval,
            trigger_immediate_cleanup = TriggerCleanup,
            add_surface = RegrowthAddSurface})

commands.add_command("trigger-map-cleanup",
    "Force immediate removal of all expired chunks (unused chunk removal mod)",
    ForceRemoveChunksCmd)

--------------------------------------------------------------------------------
-- ALL EVENT HANLDERS ARE HERE IN ONE PLACE!
--------------------------------------------------------------------------------

----------------------------------------
-- On Init - only runs once the first
--   time the game starts
----------------------------------------
script.on_init(function(event)

    -- FIRST
    InitOarcConfig()

    -- Regrowth (always init so we can enable during play.)
    RegrowthInit()
	on_init_on_tick_shedule(event)

    -- Create new game surface
    CreateGameSurface()

    -- MUST be before other stuff, but after surface creation.
    InitSpawnGlobalsAndForces()

    -- Frontier Silo Area Generation
    if (global.ocfg.frontier_rocket_silo) then
        SpawnSilosAndGenerateSiloAreas()
    end

    -- Everyone do the shuffle. Helps avoid always starting at the same location.
    global.vanillaSpawns = FYShuffle(global.vanillaSpawns)
    log("Vanilla spawns:")
    log(serpent.block(global.vanillaSpawns))
	
	game.surfaces[GAME_SURFACE_NAME].ticks_per_day = game.surfaces[GAME_SURFACE_NAME].ticks_per_day * DAYTIME_MULTIPILER
	
	if ROCK_WRECKAGE_HEAL_ENABLED then
		rock_wreckage_damaged_player_joined(event)
	end
	init_floaty_chat(event)
end)


----------------------------------------
-- Rocket launch event
-- Used for end game win conditions / unlocking late game stuff
----------------------------------------
script.on_event(defines.events.on_rocket_launched, function(event)
    if global.ocfg.frontier_rocket_silo then
        RocketLaunchEvent(event)
    end
end)

local first_chunk_generated_flag = false


----------------------------------------
-- Chunk Generation
----------------------------------------
script.on_event(defines.events.on_chunk_generated, function(event)

    if global.ocfg.enable_regrowth then
        RegrowthChunkGenerate(event)
    end
    if global.ocfg.enable_undecorator then
        UndecorateOnChunkGenerate(event)
    end

    if global.ocfg.frontier_rocket_silo then
        GenerateRocketSiloChunk(event)
    end

    SeparateSpawnsGenerateChunk(event)

    CreateHoldingPen(event.surface, event.area, 16, 32)
	
	if ENABLE_WRECKAGE_SPAWNER then
		wreckage_spawner(event)
	end
	
	if ENABLE_CRASHSITES then
		crashsite(event)
	end
	
	if MAP_TOXIPLANET then
		ToxiPlanet(event)
	end
	
	if MAP_JUNKYARD then
		Junkyard(event)
	end
	
	if MAP_JUNKYARD_V2 then
		JunkyardV2(event)
	end
	
	if MAP_REDDESERT then
		RedDesert(event)
	end
	
	if MAP_WOODY then
		Woody(event)
	end
	
	if MAP_SALTLAKE then
		SaltLake(event)
	end
	
	if ENABLE_MARKETS then
		markets(event)
	end
	if ENABLE_MIXED_ORES then
	mixed_ores(event)
	end
end)

script.on_event(defines.events.on_chunk_charted, function(event)
	if MAP_JUNKYARD then
		junkyard_chunk_charted(event)
	end
	if MAP_JUNKYARD_V2 then
		junkyard_chunk_chartedV2(event)
	end
	if MAP_NORMAL then
		normal_chunk_charted(event)
	end
	if MAP_REDDESERT then
		RedDesert_chunk_charted(event)
	end
	if MAP_TOXIPLANET then
		ToxiPlanet_chunk_charted(event)
	end
	if MAP_WOODY then
		Woody_chunk_charted(event)
	end
end)

local first_chunk_generated_flag = false

script.on_event(defines.events.on_player_mined_entity, function(event)
	if MINEABLE_WRECKAGE_YIELDS_SCRAP then
		mineable_wreckage_yields_scrap(event)
	end
	
	if ROCKS_YIELD_ORE then
		rocks_yield_ore_on_player(event)
	end
	
	if UNEARTHING_BITERS_WRECKAGE then
		UnearthingBiters_wreckage(event)
	end
	
	if JUNK_YIELD_ORE_VEINS_ENABLED then
		JunkYieldOreVeins(event)
	end
	
	if ROCKS_YIELD_COINS_ENABLE then
		rocks_yield_coins(event)
	end
	
	if WRECKAGE_YIELD_COINS_ENABLE then
		wreckage_yield_coins(event)
	end
	if ENABLE_PIRANHA then
		piranha_bite(event)
	end
	if ENABLE_SCRAP_DEFENSE then
		Scrap_defense(event)
	end
end)

script.on_event(defines.events.on_player_changed_position, function(event)
	if UNEARTHING_BITERS_NIGHT then
		UnearthingBiters_night(event)
	end
end)

----------------------------------------
-- Gui Click
----------------------------------------
script.on_event(defines.events.on_gui_click, function(event)

    -- Don't interfere with other mod related stuff.
    if (event.element.get_mod() ~= nil) then return end

    if global.ocfg.enable_tags then
        TagGuiClick(event)
    end

    if global.ocfg.enable_player_list then
        PlayerListGuiClick(event)
    end

    WelcomeTextGuiClick(event)
    SpawnOptsGuiClick(event)
    SpawnCtrlGuiClick(event)
    SharedSpwnOptsGuiClick(event)
    BuddySpawnOptsGuiClick(event)
    BuddySpawnWaitMenuClick(event)
    BuddySpawnRequestMenuClick(event)
    SharedSpawnJoinWaitMenuClick(event)

    ClickOarcGuiButton(event)

    GameOptionsGuiClick(event)
	AutostashGuiClick(event)
end)

script.on_event(defines.events.on_gui_checked_state_changed, function (event)
    SpawnOptsRadioSelect(event)
    SpawnCtrlGuiOptionsSelect(event)
end)

script.on_event(defines.events.on_gui_selected_tab_changed, function (event)
    TabChangeOarcGui(event)
end)

----------------------------------------
-- Player Events
----------------------------------------
script.on_event(defines.events.on_player_joined_game, function(event)
    PlayerJoinedMessages(event)
    ServerWriteFile("player_events", game.players[event.player_index].name .. " joined the game." .. "\n")
	
	if BITERS_DOUBLE_DAMAGE then
        biters_double_damage(event)
    end
	
	if ENABLE_HUNGER then
		create_hunger(event)
	end
	autostash_button(event)
end)

script.on_event(defines.events.on_player_created, function(event)
    local player = game.players[event.player_index]

    -- Move the player to the game surface immediately.
    player.teleport({x=0,y=0}, GAME_SURFACE_NAME)

    if global.ocfg.enable_long_reach then
        GivePlayerLongReach(player)
    end

    SeparateSpawnsPlayerCreated(event.player_index)

    InitOarcGuiTabs(player)
end)

script.on_event(defines.events.on_player_respawned, function(event)
    SeparateSpawnsPlayerRespawned(event)

    PlayerRespawnItems(event)

    if global.ocfg.enable_long_reach then
        GivePlayerLongReach(game.players[event.player_index])
    end
	
	if ENABLE_HUNGER then
		respawn_hunger(event)
	end
end)

script.on_event(defines.events.on_player_left_game, function(event)
    ServerWriteFile("player_events", game.players[event.player_index].name .. " left the game." .. "\n")
    FindUnusedSpawns(game.players[event.player_index], true)
end)

----------------------------------------
-- On BUILD entity. Don't forget on_robot_built_entity too!
----------------------------------------
script.on_event(defines.events.on_built_entity, function(event)
    if global.ocfg.enable_autofill then
        Autofill(event)
    end

    if global.ocfg.enable_regrowth then
        remote.call("oarc_regrowth",
                    "area_offlimits_tilepos",
                    GAME_SURFACE_NAME,
                    event.created_entity.position,
                    2)
    end

    if ENABLE_ANTI_GRIEFING then
        SetItemBlueprintTimeToLive(event)
    end

    if global.ocfg.frontier_rocket_silo then
        BuildSiloAttempt(event)
    end

end)

script.on_event(defines.events.on_player_used_capsule, function(event)
	if ENABLE_HUNGER then
		eat_fish(event)
	end
end)

----------------------------------------
-- On script_raised_built. This should help catch mods that
-- place items that don't count as player_built and robot_built.
-- Specifically FARL.
----------------------------------------
script.on_event(defines.events.script_raised_built, function(event)
    if global.ocfg.enable_regrowth then
        remote.call("oarc_regrowth",
                    "area_offlimits_tilepos",
                    GAME_SURFACE_NAME,
                    event.entity.position,
                    2)
    end
end)

script.on_event(defines.events.on_sector_scanned, function (event)
    if global.ocfg.enable_regrowth then
        RegrowthSectorScan(event)
    end
end)
-- script.on_event(defines.events.on_sector_scanned, function (event)

-- end)

----------------------------------------
--
----------------------------------------
script.on_event(defines.events.on_robot_built_entity, function (event)
    if global.ocfg.enable_regrowth then
        remote.call("oarc_regrowth",
                    "area_offlimits_tilepos",
                    GAME_SURFACE_NAME,
                    event.created_entity.position,
                    2)
    end
    if global.ocfg.frontier_rocket_silo then
        BuildSiloAttempt(event)
    end
end)


----------------------------------------
-- Shared chat, so you don't have to type /s
-- But you do lose your player colors across forces.
----------------------------------------
script.on_event(defines.events.on_console_chat, function(event)
    if (event.player_index) then
        ServerWriteFile("server_chat", game.players[event.player_index].name .. ": " .. event.message .. "\n")
    end
    if (global.ocfg.enable_shared_chat) then
        if (event.player_index ~= nil) then
            ShareChatBetweenForces(game.players[event.player_index], event.message)
        end
    end
	floaty_chat(event)
end)

local railgun_technology_map = {['laser-turrets'] = 'railgun'}
local railgun_dart_technology_map = {['laser-turrets'] = 'railgun-dart'}

----------------------------------------
-- On Research Finished
-- This is where you can permanently remove researched techs
----------------------------------------
script.on_event(defines.events.on_research_finished, function(event)

    -- Never allows players to build rocket-silos in "frontier" mode.
    if global.ocfg.frontier_rocket_silo and not global.ocfg.frontier_allow_build then
        RemoveRecipe(event.research.force, "rocket-silo")
    end

    if global.ocfg.lock_goodies_rocket_launch and
        (not global.satellite_sent or not global.satellite_sent[event.research.force.name]) then
        RemoveRecipe(event.research.force, "productivity-module-3")
        RemoveRecipe(event.research.force, "speed-module-3")
    end

    if global.ocfg.enable_loaders then
        EnableLoaders(event)
    end
	
	if RAILGUN_ENABLED then
        local research = event.research
        local recipe = railgun_technology_map[research.name]
        if recipe then
            research.force.recipes[recipe].enabled = true
        end
    end
	if RAILGUN_ENABLED then
        local research = event.research
        local recipe = railgun_dart_technology_map[research.name]
        if recipe then
            research.force.recipes[recipe].enabled = true
        end
    end
	if SHOTGUN_BUFF_ENABLED then
		shotgun_buff(event)
    end
end)

----------------------------------------
-- On Entity Spawned and On Biter Base Built
-- This is where I modify biter spawning based on location and other factors.
----------------------------------------
script.on_event(defines.events.on_entity_spawned, function(event)
    if (global.ocfg.modified_enemy_spawning) then
        ModifyEnemySpawnsNearPlayerStartingAreas(event)
    end
end)
script.on_event(defines.events.on_biter_base_built, function(event)
    if (global.ocfg.modified_enemy_spawning) then
        ModifyEnemySpawnsNearPlayerStartingAreas(event)
    end
end)

----------------------------------------
-- On Corpse Timed Out
-- Save player's stuff so they don't lose it if they can't get to the corpse fast enough.
----------------------------------------
script.on_event(defines.events.on_character_corpse_expired, function(event)
    DropGravestoneChestFromCorpse(event.corpse)
end)

script.on_event(defines.events.on_entity_damaged, function(event)
	if BITERS_DOUBLE_HP then
		biters_double_hp(event)
	end
	if SPAWNERS_DOUBLE_HP then
		spawners_double_hp(event)
	end
	if RAILGUN_ENABLED then
		railgun_enhancer(event)
	end
	if ROCK_WRECKAGE_HEAL_ENABLED then
		rock_wreckage_damaged(event)
	end
	if TELEPORTING_WORM_ENABLED then
		teleporting_worm(event)
	end
	if EXPLOSIVES_ARE_EXPLOSIVE_ENABLED then
		explosives_damaged_explosive(event)
	end
	wreckage_double_hp(event)
end)

----------------------------------------
-- On tick events. Stuff that needs to happen at regular intervals.
-- Delayed events, delayed spawns, ...
----------------------------------------
script.on_event(defines.events.on_tick, function(event)
    if global.ocfg.enable_regrowth then
        RegrowthOnTick()
        RegrowthForceRemovalOnTick()
    end

    DelayedSpawnOnTick()

    if global.ocfg.frontier_rocket_silo then
        DelayedSiloCreationOnTick(game.surfaces[GAME_SURFACE_NAME])
    end
	
	on_tick_shedule(event)
	
	if ENABLE_HUNGER then
		update_hunger(event)
	end
	
	if ENABLE_FISH_RESPAWNER then
		fish_respawner(event)
	end
	
	if ROCK_WRECKAGE_HEAL_ENABLED then
		rock_wreckage_damaged_tick(event)
	end
	
	if EXPLOSIVES_ARE_EXPLOSIVE_ENABLED then
		explosives_tick_explosive(event)
	end

end)

script.on_event(defines.events.on_entity_died, function(event)
	if BITERS_YIELD_COINS then
		biters_yield_coins(event)
	end
	if SPAWNERS_CONTAIN_BITERS then
		spawners_contain_biters(event)
	end
	if SPLICE then
		splice(event)
	end
	if ROCKS_YIELD_ORE then
		rocks_yield_ore_on_entity_died(event)
	end
	if SPAWNERS_CONTAIN_ACID_ENABLED then
		spawners_contain_acid(event)
	end
	if SPITTER_CONTAIN_ACID_ENABLED then
		small_spitter_contain_acid(event)
		medium_spitter_contain_acid(event)
		big_spitter_contain_acid(event)
		behemoth_spitter_contain_acid(event)
	end
end)

script.on_event(defines.events.on_player_built_tile, function(event)
	if DYNAMIC_LANDFILL then
		DynamicLandfillPlayer(event)
	end
end)

script.on_event(defines.events.on_robot_built_tile, function(event)
	if DYNAMIC_LANDFILL then
		DynamicLandfillRobot(event)
	end
end)

script.on_event(defines.events.on_marked_for_deconstruction, function(event)
	if DISABLED_DECONSTRUCTION_ENABLE then
		DisabledDeconstruction(event)
	end
end)

script.on_event(defines.events.on_market_item_purchased, function(event)
	if COIN_BANK_ENABLE then
		CoinBank(event)
	end
	
	if COIN_BOOST_ENABLE then
		CoinBoost(event)
	end
end)

script.on_event(defines.events.on_player_died, function(event)
	if MAP_CORPSE_MARKER then
		draw_player_corpse(event)
	end
end)

script.on_event(defines.events.on_character_corpse_expired, function(event)
	if MAP_CORPSE_MARKER then
		remove_expired_corpse_marker(event)
	end
end)

script.on_event(defines.events.on_pre_player_mined_item, function(event)
	if MAP_CORPSE_MARKER then
		remove_mined_corpse_marker(event)
	end
end)

script.on_event(defines.events.on_console_command, function(event)
	print_silent_command(event)
end)